//------------------------------------------------------------------
#property  copyright "free code"
#property  link      "free code"
//------------------------------------------------------------------
#property  indicator_chart_window
#property  indicator_buffers 3
#property  indicator_color1  LimeGreen
#property  indicator_color2  DarkOrange
#property  indicator_color3  DarkOrange
#property  indicator_width1  2
#property  indicator_width2  2
#property  indicator_width3  2

extern string TimeFrame = "Current time frame";
extern double FastEMA   = 12;
extern double SlowEMA   = 26;
extern string UniqueID  = "macdline1";
extern color  LineColor = Silver;
extern int    LineStyle = STYLE_DOT;
extern int    LineWidth = 1;

double macdline[];
double macdlineda[];
double macdlinedb[];
double slope[];

int      timeFrame;
bool     calculateValue;
bool     returnBars;
string   indicatorFileName;

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int init()
{
   IndicatorBuffers(4);
   SetIndexBuffer(0,macdline);
   SetIndexBuffer(1,macdlineda);
   SetIndexBuffer(2,macdlinedb);
   SetIndexBuffer(3,slope);

         //
         //
         //
         //
         //
                  
         indicatorFileName = WindowExpertName();
         calculateValue    = (TimeFrame=="CalculateValue"); if (calculateValue) return(0);
         returnBars        = (TimeFrame=="returnBars");     if (returnBars)     return(0);
         timeFrame         = stringToTimeFrame(TimeFrame);
         
         //
         //
         //
         //
         //

   return(0);
}
void deinit()
{
   ObjectDelete(UniqueID);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
           int limit=MathMin(Bars-counted_bars,Bars-2);
           if (returnBars) { macdline[0] = limit+1; return(0); }

   //
   //
   //
   //
   //
   
   if (calculateValue || timeFrame==Period())
   {
      double x1=2/(FastEMA+1);
      double x2=2/(SlowEMA+1);
      if (slope[limit]==-1) CleanPoint(limit,macdlineda,macdlinedb);
      for (int i=limit; i>=0; i--) 
      {
         double fema = iMA(NULL,0,FastEMA,0,MODE_EMA,PRICE_CLOSE,i+1);
         double sema = iMA(NULL,0,SlowEMA,0,MODE_EMA,PRICE_CLOSE,i+1);
         double dif1 = iMACD(NULL,0,FastEMA,SlowEMA,9,PRICE_CLOSE,MODE_MAIN,i+1);
            macdline[i]   = (dif1+fema*x1-fema-sema*x2+sema)/(x1-x2);
            macdlineda[i] = EMPTY_VALUE;
            macdlinedb[i] = EMPTY_VALUE;
                 slope[i] = slope[i+1];
                 if (macdline[i]>macdline[i+1]) slope[i] =  1;
                 if (macdline[i]<macdline[i+1]) slope[i] = -1;
                 if (slope[i]==-1) PlotPoint(i,macdlineda,macdlinedb,macdline);
      }
      if (LineWidth>-1)
      {
         ObjectCreate(UniqueID,OBJ_HLINE,0,0,0);
            ObjectSet(UniqueID,OBJPROP_PRICE1,macdline[0]);
            ObjectSet(UniqueID,OBJPROP_COLOR,LineColor);
            ObjectSet(UniqueID,OBJPROP_STYLE,LineStyle);
            ObjectSet(UniqueID,OBJPROP_WIDTH,LineWidth);
      }         
      return(0);
   }
   
   //
   //
   //
   //
   //
   
   limit = MathMax(limit,MathMin(Bars,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   if (slope[limit]==-1) CleanPoint(limit,macdlineda,macdlinedb);
   for (i=limit;i>=0;i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         macdlineda[i] = EMPTY_VALUE;
         macdlinedb[i] = EMPTY_VALUE;
         macdline[i]   = iCustom(NULL,timeFrame,indicatorFileName,"CalculateValue",FastEMA,SlowEMA,UniqueID,LineColor,LineStyle,LineWidth,0,y);
         slope[i]      = iCustom(NULL,timeFrame,indicatorFileName,"CalculateValue",FastEMA,SlowEMA,UniqueID,LineColor,LineStyle,LineWidth,3,y);
   }
   for (i=limit;i>=0;i--) if (slope[i]==-1) PlotPoint(i,macdlineda,macdlinedb,macdline);
   return(0);         
   
   
}   

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

void CleanPoint(int i,double& first[],double& second[])
{
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}

//
//
//
//
//

void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (first[i+1] == EMPTY_VALUE)
      {
         if (first[i+2] == EMPTY_VALUE) {
                first[i]   = from[i];
                first[i+1] = from[i+1];
                second[i]  = EMPTY_VALUE;
            }
         else {
                second[i]   =  from[i];
                second[i+1] =  from[i+1];
                first[i]    = EMPTY_VALUE;
            }
      }
   else
      {
         first[i]  = from[i];
         second[i] = EMPTY_VALUE;
      }
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}